﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using log4net;
using Newtonsoft.Json;
using Triton.Bot.Settings;
using Triton.Common;
using Triton.Game.Mapping;
using Logger = Triton.Common.LogUtilities.Logger;

namespace Quest
{
    /// <summary>Settings for the Quest plugin. </summary>
    public class QuestSettings : JsonSettings
    {
        private static readonly ILog Log = Logger.GetLoggerInstanceForType();

        private static QuestSettings _instance;

        /// <summary>The current instance for this class. </summary>
        public static QuestSettings Instance
        {
            get { return _instance ?? (_instance = new QuestSettings()); }
        }

        /// <summary>The default ctor. Will use the settings path "Quest".</summary>
        public QuestSettings()
            : base(GetSettingsFilePath(Configuration.Instance.Name, string.Format("{0}.json", "Quest")))
        {
        }

        private bool _stopAfterAllQuestsAreDone;
        private bool _onlyUseCustomDecks;
        private bool _exitAfterAllQuestsAreDone;

        /// <summary>
        /// Should the plugin stop the bot after all quests are done?
        /// </summary>
        [DefaultValue(true)]
        public bool StopAfterAllQuestsAreDone
        {
            get { return _stopAfterAllQuestsAreDone; }
            set
            {
                if (!value.Equals(_stopAfterAllQuestsAreDone))
                {
                    _stopAfterAllQuestsAreDone = value;
                    NotifyPropertyChanged(() => StopAfterAllQuestsAreDone);
                }
                Log.InfoFormat("[QuestSettings] StopAfterAllQuestsAreDone = {0}.", _stopAfterAllQuestsAreDone);
            }
        }

        /// <summary>
        /// Should the plugin only use custom decks to complete quests?
        /// </summary>
        [DefaultValue(false)]
        public bool OnlyUseCustomDecks
        {
            get { return _onlyUseCustomDecks; }
            set
            {
                if (!value.Equals(_onlyUseCustomDecks))
                {
                    _onlyUseCustomDecks = value;
                    NotifyPropertyChanged(() => OnlyUseCustomDecks);
                }
                Log.InfoFormat("[QuestSettings] OnlyUseCustomDecks = {0}.", _onlyUseCustomDecks);
            }
        }


        /// <summary>
        /// Should the plugin exit the bot after all quests are done?
        /// </summary>
        [DefaultValue(true)]
        public bool ExitAfterAllQuestsAreDone
        {
            get { return _exitAfterAllQuestsAreDone; }
            set
            {
                if (!value.Equals(_exitAfterAllQuestsAreDone))
                {
                    _exitAfterAllQuestsAreDone = value;
                    NotifyPropertyChanged(() => ExitAfterAllQuestsAreDone);
                }
                Log.InfoFormat("[QuestSettings] ExitAfterAllQuestsAreDone = {0}.", _exitAfterAllQuestsAreDone);
            }
        }
    }
}
